import csv

from django.contrib.auth.mixins import PermissionRequiredMixin
from django.http import HttpResponse
from django.views.generic import View

from core.models import StudentInfo, SubmissionType
from core.permissions import IsReviewer


class StudentCSVExport(PermissionRequiredMixin, View):

    def has_permission(self):
        return IsReviewer().has_permission(self.request, self)

    def get(self, request, format=None):
        # Create the HttpResponse object with the appropriate CSV header.
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="results.csv"'

        writer = csv.writer(response)
        writer.writerow(['Matrikel',
                         'Username',
                         'Name',
                         'Sum',
                         *SubmissionType.objects
                         .order_by('name')
                         .values_list('name', flat=True)])

        for student in StudentInfo.get_annotated_score_submission_list():
            writer.writerow([
                student.matrikel_no,
                student.user.username,
                student.user.fullname,
                student.overall_score,
                *student.score_per_submission().values()
            ])

        return response
